@@ -4,11 +4,13 @@ use alloc::fmt;
 
 const MAX_OID_LENGTH: usize = 63;
 
-/// Represents an ASN.1 `OBJECT IDENTIFIER`. ObjectIdentifiers are opaque, the only thing may be
-/// done with them is test if they are equal to another `ObjectIdentifier`. The generally
-/// recommended practice for handling them is to create some `ObjectIdentifier` constants with
-/// `asn1::oid!()` and then compare ObjectIdentifiers you get from parsing to
-/// those.
+/// Represents an ASN.1 `OBJECT IDENTIFIER`.
+///
+/// `ObjectIdentifier`s are opaque, the only thing may be done with them is
+/// test if they are equal to another `ObjectIdentifier`. The generally
+/// recommended practice for handling them is to create some
+/// `ObjectIdentifier` constants with `asn1::oid!()` and then compare them
+/// with `ObjectIdentifier`s you get from parsing.
 ///
 /// `asn1::oid!()` takes a series of arcs, for example: `asn1::oid!(1, 2, 3)`.
 ///
@@ -28,8 +30,8 @@ impl ObjectIdentifier {
     pub fn from_string(oid: &str) -> Option<ObjectIdentifier> {
         let mut parts = oid.split('.');
 
-        let first = parts.next()?.parse::<u32>().ok()?;
-        let second = parts.next()?.parse::<u32>().ok()?;
+        let first = parts.next()?.parse::<u128>().ok()?;
+        let second = parts.next()?.parse::<u128>().ok()?;
         if first > 2 || (first < 2 && second >= 40) {
             return None;
         }
@@ -41,7 +43,7 @@ impl ObjectIdentifier {
         for part in parts {
             der_data_len += base128::write_base128_int(
                 &mut der_data[der_data_len..],
-                part.parse::<u32>().ok()?,
+                part.parse::<u128>().ok()?,
             )?;
         }
         Some(ObjectIdentifier {
@@ -61,7 +63,10 @@ impl ObjectIdentifier {
 
         let mut parsed = (0, data);
         while !parsed.1.is_empty() {
-            parsed = base128::read_base128_int(parsed.1)?;
+            // `base128::read_base128_int` can return a `ShortData` error, but
+            // in context here that means `InvalidValue`.
+            parsed = base128::read_base128_int(parsed.1)
+                .map_err(|_| ParseError::new(ParseErrorKind::InvalidValue))?;
         }
 
         let mut storage = [0; MAX_OID_LENGTH];
@@ -132,6 +137,7 @@ mod tests {
     use super::MAX_OID_LENGTH;
     use crate::{ObjectIdentifier, ParseError, ParseErrorKind};
     use alloc::format;
+    #[cfg(not(feature = "std"))]
     use alloc::string::ToString;
 
     #[test]
@@ -159,6 +165,9 @@ mod tests {
             "1.2.3.4",
             "1.2.840.133549.1.1.5",
             "2.100.3",
+            "2.1.750304883",
+            "2.25.223663413560230117710484359924050447509",
+            "2.25.340282366920938463463374607431768211455",
         ] {
             assert!(ObjectIdentifier::from_string(val).is_some());
         }
@@ -198,6 +207,8 @@ mod tests {
             "1.2.840.133549.1.1.5",
             "2.100.3",
             "2.1.750304883",
+            "2.25.223663413560230117710484359924050447509",
+            "2.25.340282366920938463463374607431768211455",
         ] {
             assert_eq!(
                 &ObjectIdentifier::from_string(val).unwrap().to_string(),
