@@ -138,13 +138,20 @@ impl<'a> Asn1Readable<'a> for Tlv<'a> {
         true
     }
 }
-impl<'a> Asn1Writable for Tlv<'a> {
+impl Asn1Writable for Tlv<'_> {
     #[inline]
     fn write(&self, w: &mut Writer<'_>) -> WriteResult {
         w.write_tlv(self.tag, move |dest| dest.push_slice(self.data))
     }
 }
 
+impl Asn1Writable for &Tlv<'_> {
+    #[inline]
+    fn write(&self, w: &mut Writer<'_>) -> WriteResult {
+        Tlv::write(self, w)
+    }
+}
+
 /// The ASN.1 NULL type, for use with `Parser.read_element` and
 /// `Writer.write_element`.
 pub type Null = ();
@@ -198,7 +205,7 @@ impl<'a> SimpleAsn1Readable<'a> for &'a [u8] {
     }
 }
 
-impl<'a> SimpleAsn1Writable for &'a [u8] {
+impl SimpleAsn1Writable for &[u8] {
     const TAG: Tag = Tag::primitive(0x04);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self)
@@ -317,7 +324,7 @@ impl<'a> SimpleAsn1Readable<'a> for PrintableString<'a> {
     }
 }
 
-impl<'a> SimpleAsn1Writable for PrintableString<'a> {
+impl SimpleAsn1Writable for PrintableString<'_> {
     const TAG: Tag = Tag::primitive(0x13);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.0.as_bytes())
@@ -366,7 +373,7 @@ impl<'a> SimpleAsn1Readable<'a> for IA5String<'a> {
         IA5String::new_from_bytes(data).ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for IA5String<'a> {
+impl SimpleAsn1Writable for IA5String<'_> {
     const TAG: Tag = Tag::primitive(0x16);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.0.as_bytes())
@@ -399,7 +406,7 @@ impl<'a> SimpleAsn1Readable<'a> for Utf8String<'a> {
             .ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for Utf8String<'a> {
+impl SimpleAsn1Writable for Utf8String<'_> {
     const TAG: Tag = Tag::primitive(0x0c);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.0.as_bytes())
@@ -454,7 +461,7 @@ impl<'a> SimpleAsn1Readable<'a> for VisibleString<'a> {
             .ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for VisibleString<'a> {
+impl SimpleAsn1Writable for VisibleString<'_> {
     const TAG: Tag = Tag::primitive(0x1a);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.0.as_bytes())
@@ -504,7 +511,7 @@ impl<'a> SimpleAsn1Readable<'a> for BMPString<'a> {
         BMPString::new(data).ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for BMPString<'a> {
+impl SimpleAsn1Writable for BMPString<'_> {
     const TAG: Tag = Tag::primitive(0x1e);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.as_utf16_be_bytes())
@@ -554,7 +561,7 @@ impl<'a> SimpleAsn1Readable<'a> for UniversalString<'a> {
         UniversalString::new(data).ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for UniversalString<'a> {
+impl SimpleAsn1Writable for UniversalString<'_> {
     const TAG: Tag = Tag::primitive(0x1c);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.as_utf32_be_bytes())
@@ -666,7 +673,7 @@ impl<'a> SimpleAsn1Readable<'a> for BigUint<'a> {
         BigUint::new(data).ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for BigUint<'a> {
+impl SimpleAsn1Writable for BigUint<'_> {
     const TAG: Tag = Tag::primitive(0x02);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.as_bytes())
@@ -746,7 +753,7 @@ impl<'a> SimpleAsn1Readable<'a> for BigInt<'a> {
         BigInt::new(data).ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for BigInt<'a> {
+impl SimpleAsn1Writable for BigInt<'_> {
     const TAG: Tag = Tag::primitive(0x02);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_slice(self.as_bytes())
@@ -818,7 +825,7 @@ impl<'a> SimpleAsn1Readable<'a> for BitString<'a> {
             .ok_or_else(|| ParseError::new(ParseErrorKind::InvalidValue))
     }
 }
-impl<'a> SimpleAsn1Writable for BitString<'a> {
+impl SimpleAsn1Writable for BitString<'_> {
     const TAG: Tag = Tag::primitive(0x03);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         dest.push_byte(self.padding_bits())?;
@@ -914,7 +921,8 @@ fn push_four_digits(dest: &mut WriteBuf, val: u16) -> WriteResult {
 }
 
 /// A structure representing a (UTC timezone) date and time.
-/// Wrapped by `UtcTime` and `GeneralizedTime`.
+/// Wrapped by `UtcTime` and `X509GeneralizedTime` and used in
+/// `GeneralizedTime`.
 #[derive(Debug, Clone, PartialEq, Hash, Eq, PartialOrd)]
 pub struct DateTime {
     year: u16,
@@ -1039,14 +1047,14 @@ impl SimpleAsn1Writable for UtcTime {
     }
 }
 
-/// Used for parsing and writing ASN.1 `GENERALIZED TIME` values. Wraps a
-/// `DateTime`.
+/// Used for parsing and writing ASN.1 `GENERALIZED TIME` values used in X.509.
+/// Wraps a `DateTime`.
 #[derive(Debug, Clone, PartialEq, Hash, Eq)]
-pub struct GeneralizedTime(DateTime);
+pub struct X509GeneralizedTime(DateTime);
 
-impl GeneralizedTime {
-    pub fn new(dt: DateTime) -> ParseResult<GeneralizedTime> {
-        Ok(GeneralizedTime(dt))
+impl X509GeneralizedTime {
+    pub fn new(dt: DateTime) -> ParseResult<X509GeneralizedTime> {
+        Ok(X509GeneralizedTime(dt))
     }
 
     pub fn as_datetime(&self) -> &DateTime {
@@ -1054,6 +1062,108 @@ impl GeneralizedTime {
     }
 }
 
+impl SimpleAsn1Readable<'_> for X509GeneralizedTime {
+    const TAG: Tag = Tag::primitive(0x18);
+    fn parse_data(mut data: &[u8]) -> ParseResult<X509GeneralizedTime> {
+        let year = read_4_digits(&mut data)?;
+        let month = read_2_digits(&mut data)?;
+        let day = read_2_digits(&mut data)?;
+        let hour = read_2_digits(&mut data)?;
+        let minute = read_2_digits(&mut data)?;
+        let second = read_2_digits(&mut data)?;
+
+        // Fractionals are forbidden (RFC5280)
+
+        read_tz_and_finish(&mut data)?;
+
+        X509GeneralizedTime::new(DateTime::new(year, month, day, hour, minute, second)?)
+    }
+}
+
+impl SimpleAsn1Writable for X509GeneralizedTime {
+    const TAG: Tag = Tag::primitive(0x18);
+    fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
+        let dt = self.as_datetime();
+        push_four_digits(dest, dt.year())?;
+        push_two_digits(dest, dt.month())?;
+        push_two_digits(dest, dt.day())?;
+
+        push_two_digits(dest, dt.hour())?;
+        push_two_digits(dest, dt.minute())?;
+        push_two_digits(dest, dt.second())?;
+
+        dest.push_byte(b'Z')
+    }
+}
+
+/// Used for parsing and writing ASN.1 `GENERALIZED TIME` values,
+/// including values with fractional seconds of up to nanosecond
+/// precision.
+#[derive(Debug, Clone, PartialEq, PartialOrd, Hash, Eq)]
+pub struct GeneralizedTime {
+    datetime: DateTime,
+    nanoseconds: Option<u32>,
+}
+
+impl GeneralizedTime {
+    pub fn new(dt: DateTime, nanoseconds: Option<u32>) -> ParseResult<GeneralizedTime> {
+        if let Some(val) = nanoseconds {
+            if val < 1 || val >= 1e9 as u32 {
+                return Err(ParseError::new(ParseErrorKind::InvalidValue));
+            }
+        }
+
+        Ok(GeneralizedTime {
+            datetime: dt,
+            nanoseconds,
+        })
+    }
+
+    pub fn as_datetime(&self) -> &DateTime {
+        &self.datetime
+    }
+
+    pub fn nanoseconds(&self) -> Option<u32> {
+        self.nanoseconds
+    }
+}
+
+fn read_fractional_time(data: &mut &[u8]) -> ParseResult<Option<u32>> {
+    // We cannot use read_byte here because it will advance the pointer
+    // However, we know that the is suffixed by 'Z' so reading into an empty
+    // data should lead to an error.
+    if data.first() == Some(&b'.') {
+        *data = &data[1..];
+
+        let mut fraction = 0u32;
+        let mut digits = 0;
+        // Read up to 9 digits
+        for b in data.iter().take(9) {
+            if !b.is_ascii_digit() {
+                if digits == 0 {
+                    // We must have at least one digit
+                    return Err(ParseError::new(ParseErrorKind::InvalidValue));
+                }
+                break;
+            }
+            fraction = fraction * 10 + (b - b'0') as u32;
+            digits += 1;
+        }
+        *data = &data[digits..];
+
+        // No trailing zero
+        if fraction % 10 == 0 {
+            return Err(ParseError::new(ParseErrorKind::InvalidValue));
+        }
+
+        // Now let scale up in nanoseconds
+        let nanoseconds: u32 = fraction * 10u32.pow(9 - digits as u32);
+        Ok(Some(nanoseconds))
+    } else {
+        Ok(None)
+    }
+}
+
 impl SimpleAsn1Readable<'_> for GeneralizedTime {
     const TAG: Tag = Tag::primitive(0x18);
     fn parse_data(mut data: &[u8]) -> ParseResult<GeneralizedTime> {
@@ -1064,9 +1174,13 @@ impl SimpleAsn1Readable<'_> for GeneralizedTime {
         let minute = read_2_digits(&mut data)?;
         let second = read_2_digits(&mut data)?;
 
+        let fraction = read_fractional_time(&mut data)?;
         read_tz_and_finish(&mut data)?;
 
-        GeneralizedTime::new(DateTime::new(year, month, day, hour, minute, second)?)
+        GeneralizedTime::new(
+            DateTime::new(year, month, day, hour, minute, second)?,
+            fraction,
+        )
     }
 }
 
@@ -1082,6 +1196,21 @@ impl SimpleAsn1Writable for GeneralizedTime {
         push_two_digits(dest, dt.minute())?;
         push_two_digits(dest, dt.second())?;
 
+        if let Some(nanoseconds) = self.nanoseconds() {
+            dest.push_byte(b'.')?;
+
+            let mut buf = itoa::Buffer::new();
+            let nanos = buf.format(nanoseconds);
+            let pad = 9 - nanos.len();
+            let nanos = nanos.trim_end_matches('0');
+
+            for _ in 0..pad {
+                dest.push_byte(b'0')?;
+            }
+
+            dest.push_slice(nanos.as_bytes())?;
+        }
+
         dest.push_byte(b'Z')
     }
 }
@@ -1202,9 +1331,12 @@ declare_choice!(Choice1 => (T1 ChoiceA));
 declare_choice!(Choice2 => (T1 ChoiceA), (T2 ChoiceB));
 declare_choice!(Choice3 => (T1 ChoiceA), (T2 ChoiceB), (T3 ChoiceC));
 
-/// Represents an ASN.1 `SEQUENCE`. By itself, this merely indicates a sequence of bytes that are
-/// claimed to form an ASN1 sequence. In almost any circumstance, you'll want to immediately call
-/// `Sequence.parse` on this value to decode the actual contents therein.
+/// Represents an ASN.1 `SEQUENCE`.
+///
+/// By itself, this merely indicates a sequence of bytes that are claimed to
+// form an ASN1 sequence. In almost any circumstance, you'll want to
+/// immediately call `Sequence.parse` on this value to decode the actual
+/// contents therein.
 #[derive(Debug, PartialEq, Hash, Clone, Eq)]
 pub struct Sequence<'a> {
     data: &'a [u8],
@@ -1233,7 +1365,7 @@ impl<'a> SimpleAsn1Readable<'a> for Sequence<'a> {
         Ok(Sequence::new(data))
     }
 }
-impl<'a> SimpleAsn1Writable for Sequence<'a> {
+impl SimpleAsn1Writable for Sequence<'_> {
     const TAG: Tag = Tag::constructed(0x10);
     #[inline]
     fn write_data(&self, data: &mut WriteBuf) -> WriteResult {
@@ -1254,7 +1386,7 @@ impl<'a> SequenceWriter<'a> {
     }
 }
 
-impl<'a> SimpleAsn1Writable for SequenceWriter<'a> {
+impl SimpleAsn1Writable for SequenceWriter<'_> {
     const TAG: Tag = Tag::constructed(0x10);
     #[inline]
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
@@ -1266,7 +1398,7 @@ impl<'a> SimpleAsn1Writable for SequenceWriter<'a> {
 /// are decoded.
 pub struct SequenceOf<
     'a,
-    T: Asn1Readable<'a>,
+    T,
     const MINIMUM_LEN: usize = 0,
     const MAXIMUM_LEN: usize = { usize::MAX },
 > {
@@ -1409,12 +1541,12 @@ impl<
 
 /// Writes a `SEQUENCE OF` ASN.1 structure from a slice of `T`.
 #[derive(Hash, PartialEq, Eq, Clone)]
-pub struct SequenceOfWriter<'a, T: Asn1Writable, V: Borrow<[T]> = &'a [T]> {
+pub struct SequenceOfWriter<'a, T, V: Borrow<[T]> = &'a [T]> {
     vals: V,
     _phantom: PhantomData<&'a T>,
 }
 
-impl<'a, T: Asn1Writable, V: Borrow<[T]>> SequenceOfWriter<'a, T, V> {
+impl<T, V: Borrow<[T]>> SequenceOfWriter<'_, T, V> {
     pub fn new(vals: V) -> Self {
         SequenceOfWriter {
             vals,
@@ -1423,7 +1555,7 @@ impl<'a, T: Asn1Writable, V: Borrow<[T]>> SequenceOfWriter<'a, T, V> {
     }
 }
 
-impl<'a, T: Asn1Writable, V: Borrow<[T]>> SimpleAsn1Writable for SequenceOfWriter<'a, T, V> {
+impl<T: Asn1Writable, V: Borrow<[T]>> SimpleAsn1Writable for SequenceOfWriter<'_, T, V> {
     const TAG: Tag = Tag::constructed(0x10);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         let mut w = Writer::new(dest);
@@ -1437,7 +1569,7 @@ impl<'a, T: Asn1Writable, V: Borrow<[T]>> SimpleAsn1Writable for SequenceOfWrite
 
 /// Represents an ASN.1 `SET OF`. This is an `Iterator` over values that
 /// are decoded.
-pub struct SetOf<'a, T: Asn1Readable<'a>> {
+pub struct SetOf<'a, T> {
     parser: Parser<'a>,
     _phantom: PhantomData<T>,
 }
@@ -1550,12 +1682,12 @@ impl<'a, T: Asn1Readable<'a> + Asn1Writable> SimpleAsn1Writable for SetOf<'a, T>
 /// Writes an ASN.1 `SET OF` whose contents is a slice of `T`. This type handles
 /// ensuring that the values are properly ordered when written as DER.
 #[derive(Hash, PartialEq, Eq, Clone)]
-pub struct SetOfWriter<'a, T: Asn1Writable, V: Borrow<[T]> = &'a [T]> {
+pub struct SetOfWriter<'a, T, V: Borrow<[T]> = &'a [T]> {
     vals: V,
     _phantom: PhantomData<&'a T>,
 }
 
-impl<'a, T: Asn1Writable, V: Borrow<[T]>> SetOfWriter<'a, T, V> {
+impl<T: Asn1Writable, V: Borrow<[T]>> SetOfWriter<'_, T, V> {
     pub fn new(vals: V) -> Self {
         SetOfWriter {
             vals,
@@ -1564,7 +1696,7 @@ impl<'a, T: Asn1Writable, V: Borrow<[T]>> SetOfWriter<'a, T, V> {
     }
 }
 
-impl<'a, T: Asn1Writable, V: Borrow<[T]>> SimpleAsn1Writable for SetOfWriter<'a, T, V> {
+impl<T: Asn1Writable, V: Borrow<[T]>> SimpleAsn1Writable for SetOfWriter<'_, T, V> {
     const TAG: Tag = Tag::constructed(0x11);
     fn write_data(&self, dest: &mut WriteBuf) -> WriteResult {
         let vals = self.vals.borrow();
@@ -1717,13 +1849,28 @@ impl<T> DefinedByMarker<T> {
     }
 }
 
+impl<'a, T: Asn1Readable<'a>> Asn1Readable<'a> for DefinedByMarker<T> {
+    fn parse(_: &mut Parser<'a>) -> ParseResult<Self> {
+        panic!("parse() should never be called on a DefinedByMarker")
+    }
+    fn can_parse(_: Tag) -> bool {
+        panic!("can_parse() should never be called on a DefinedByMarker")
+    }
+}
+
+impl<T: Asn1Writable> Asn1Writable for DefinedByMarker<T> {
+    fn write(&self, _: &mut Writer<'_>) -> WriteResult {
+        panic!("write() should never be called on a DefinedByMarker")
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::{
-        parse_single, BigInt, BigUint, DateTime, DefinedByMarker, Enumerated, GeneralizedTime,
-        IA5String, ObjectIdentifier, OctetStringEncoded, OwnedBigInt, OwnedBigUint, ParseError,
-        ParseErrorKind, PrintableString, SequenceOf, SequenceOfWriter, SetOf, SetOfWriter, Tag,
-        Tlv, UtcTime, Utf8String, VisibleString,
+        parse_single, Asn1Readable, Asn1Writable, BigInt, BigUint, DateTime, DefinedByMarker,
+        Enumerated, GeneralizedTime, IA5String, ObjectIdentifier, OctetStringEncoded, OwnedBigInt,
+        OwnedBigUint, ParseError, ParseErrorKind, PrintableString, SequenceOf, SequenceOfWriter,
+        SetOf, SetOfWriter, Tag, Tlv, UtcTime, Utf8String, VisibleString, X509GeneralizedTime,
     };
     use crate::{Explicit, Implicit};
     #[cfg(not(feature = "std"))]
@@ -1995,9 +2142,58 @@ mod tests {
         assert!(UtcTime::new(DateTime::new(2100, 1, 1, 12, 0, 0).unwrap()).is_err());
     }
 
+    #[test]
+    fn test_x509_generalizedtime_new() {
+        assert!(X509GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap()).is_ok());
+    }
+
     #[test]
     fn test_generalized_time_new() {
-        assert!(GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap()).is_ok());
+        assert!(
+            GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(), Some(1234))
+                .is_ok()
+        );
+        assert!(
+            GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(), None).is_ok()
+        );
+        // Maximum fractional time is 999,999,999 nanos.
+        assert!(GeneralizedTime::new(
+            DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(),
+            Some(999_999_999_u32)
+        )
+        .is_ok());
+        assert!(GeneralizedTime::new(
+            DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(),
+            Some(1e9 as u32)
+        )
+        .is_err());
+        assert!(GeneralizedTime::new(
+            DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(),
+            Some(1e9 as u32 + 1)
+        )
+        .is_err());
+    }
+
+    #[test]
+    fn test_generalized_time_partial_ord() {
+        let point =
+            GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(), Some(1234))
+                .unwrap();
+        assert!(
+            point
+                < GeneralizedTime::new(DateTime::new(2023, 6, 30, 23, 59, 59).unwrap(), Some(1234))
+                    .unwrap()
+        );
+        assert!(
+            point
+                < GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(), Some(1235))
+                    .unwrap()
+        );
+        assert!(
+            point
+                > GeneralizedTime::new(DateTime::new(2015, 6, 30, 23, 59, 59).unwrap(), None)
+                    .unwrap()
+        );
     }
 
     #[test]
@@ -2019,4 +2215,22 @@ mod tests {
     fn test_const() {
         const _: DefinedByMarker<ObjectIdentifier> = DefinedByMarker::marker();
     }
+
+    #[test]
+    #[should_panic]
+    fn test_defined_by_marker_parse() {
+        crate::parse(b"", DefinedByMarker::<ObjectIdentifier>::parse).unwrap();
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_defined_by_marker_can_parse() {
+        DefinedByMarker::<ObjectIdentifier>::can_parse(Tag::primitive(2));
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_defined_by_marker_write() {
+        crate::write(|w| DefinedByMarker::<ObjectIdentifier>::marker().write(w)).unwrap();
+    }
 }
