@@ -1,19 +1,26 @@
-use crate::types::{Asn1Writable, SimpleAsn1Writable};
+use crate::types::Asn1Writable;
 use crate::Tag;
-use alloc::vec;
+#[cfg(not(feature = "std"))]
 use alloc::vec::Vec;
+use alloc::{fmt, vec};
 
 /// `WriteError` are returned when there is an error writing the ASN.1 data.
-///
-/// Note that `AllocationError` (and thus `WriteError` as a whole) is only
-/// produced when the `fallible-allocations` feature is used. It's expected
-/// that in the future that as this crate's MSRV increases and Rust's support
-/// for fallible allocations improves that this will be the default.
 #[derive(PartialEq, Eq, Debug)]
 pub enum WriteError {
     AllocationError,
 }
 
+impl fmt::Display for WriteError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            WriteError::AllocationError => write!(f, "allocation error"),
+        }
+    }
+}
+
+#[cfg(feature = "std")]
+impl std::error::Error for WriteError {}
+
 pub type WriteResult<T = ()> = Result<T, WriteError>;
 
 pub struct WriteBuf(Vec<u8>);
@@ -41,7 +48,6 @@ impl WriteBuf {
 
     #[inline]
     pub fn push_byte(&mut self, b: u8) -> WriteResult {
-        #[cfg(feature = "fallible-allocattions")]
         self.0
             .try_reserve(1)
             .map_err(|_| WriteError::AllocationError)?;
@@ -52,7 +58,6 @@ impl WriteBuf {
 
     #[inline]
     pub fn push_slice(&mut self, data: &[u8]) -> WriteResult {
-        #[cfg(feature = "fallible-allocattions")]
         self.0
             .try_reserve(data.len())
             .map_err(|_| WriteError::AllocationError)?;
@@ -71,6 +76,7 @@ fn _length_length(length: usize) -> u8 {
     }
     num_bytes
 }
+
 fn _insert_at_position(buf: &mut WriteBuf, pos: usize, data: &[u8]) -> WriteResult {
     for _ in 0..data.len() {
         buf.push_byte(0)?;
@@ -91,7 +97,7 @@ pub struct Writer<'a> {
 impl Writer<'_> {
     #[inline]
     #[doc(hidden)]
-    pub fn new(buf: &mut WriteBuf) -> Writer {
+    pub fn new(buf: &mut WriteBuf) -> Writer<'_> {
         Writer { buf }
     }
 
@@ -101,54 +107,6 @@ impl Writer<'_> {
         val.write(self)
     }
 
-    /// This is an alias for `write_element::<Explicit<T, tag>>` for use when
-    /// MSRV is <1.51.
-    pub fn write_explicit_element<T: Asn1Writable>(&mut self, val: &T, tag: u32) -> WriteResult {
-        let tag = crate::explicit_tag(tag);
-        self.write_tlv(tag, |dest| Writer::new(dest).write_element(val))
-    }
-
-    /// This is an alias for `write_element::<Option<Explicit<T, tag>>>` for
-    /// use when MSRV is <1.51.
-    pub fn write_optional_explicit_element<T: Asn1Writable>(
-        &mut self,
-        val: &Option<T>,
-        tag: u32,
-    ) -> WriteResult {
-        if let Some(v) = val {
-            let tag = crate::explicit_tag(tag);
-            self.write_tlv(tag, |dest| Writer::new(dest).write_element(v))
-        } else {
-            Ok(())
-        }
-    }
-
-    /// This is an alias for `write_element::<Implicit<T, tag>>` for use when
-    /// MSRV is <1.51.
-    pub fn write_implicit_element<T: SimpleAsn1Writable>(
-        &mut self,
-        val: &T,
-        tag: u32,
-    ) -> WriteResult {
-        let tag = crate::implicit_tag(tag, T::TAG);
-        self.write_tlv(tag, |dest| val.write_data(dest))
-    }
-
-    /// This is an alias for `write_element::<Option<Implicit<T, tag>>>` for
-    /// use when MSRV is <1.51.
-    pub fn write_optional_implicit_element<T: SimpleAsn1Writable>(
-        &mut self,
-        val: &Option<T>,
-        tag: u32,
-    ) -> WriteResult {
-        if let Some(v) = val {
-            let tag = crate::implicit_tag(tag, T::TAG);
-            self.write_tlv(tag, |dest| v.write_data(dest))
-        } else {
-            Ok(())
-        }
-    }
-
     /// Writes a TLV with the specified tag where the value is any bytes
     /// written to the `Vec` in the callback. The length portion of the
     /// TLV is automatically computed.
@@ -164,6 +122,11 @@ impl Writer<'_> {
         self.buf.push_byte(0)?;
         let start_len = self.buf.len();
         body(self.buf)?;
+        self.insert_length(start_len)
+    }
+
+    #[inline]
+    fn insert_length(&mut self, start_len: usize) -> WriteResult {
         let added_len = self.buf.len() - start_len;
         if added_len >= 128 {
             let n = _length_length(added_len);
@@ -184,7 +147,7 @@ impl Writer<'_> {
 /// Constructs a writer and invokes a callback which writes ASN.1 elements into
 /// the writer, then returns the generated DER bytes.
 #[inline]
-pub fn write<F: Fn(&mut Writer) -> WriteResult>(f: F) -> WriteResult<Vec<u8>> {
+pub fn write<F: Fn(&mut Writer<'_>) -> WriteResult>(f: F) -> WriteResult<Vec<u8>> {
     let mut v = WriteBuf::new(vec![]);
     let mut w = Writer::new(&mut v);
     f(&mut w)?;
@@ -200,20 +163,22 @@ pub fn write_single<T: Asn1Writable>(v: &T) -> WriteResult<Vec<u8>> {
 
 #[cfg(test)]
 mod tests {
+    #[cfg(not(feature = "std"))]
+    use alloc::boxed::Box;
+    #[cfg(not(feature = "std"))]
     use alloc::vec;
 
-    use chrono::{TimeZone, Utc};
-
     use super::{_insert_at_position, write, write_single, WriteBuf, Writer};
     use crate::types::Asn1Writable;
     use crate::{
-        parse_single, BMPString, BigInt, BigUint, BitString, Choice1, Choice2, Choice3, Enumerated,
-        GeneralizedTime, IA5String, ObjectIdentifier, OwnedBitString, PrintableString, Sequence,
+        parse_single, BMPString, BigInt, BigUint, BitString, Choice1, Choice2, Choice3, DateTime,
+        Enumerated, Explicit, GeneralizedTime, IA5String, Implicit, ObjectIdentifier,
+        OctetStringEncoded, OwnedBigInt, OwnedBigUint, OwnedBitString, PrintableString, Sequence,
         SequenceOf, SequenceOfWriter, SequenceWriter, SetOf, SetOfWriter, Tlv, UniversalString,
-        UtcTime, Utf8String, VisibleString, WriteError,
+        UtcTime, Utf8String, VisibleString, WriteError, X509GeneralizedTime,
     };
-    #[cfg(feature = "const-generics")]
-    use crate::{Explicit, Implicit};
+    #[cfg(not(feature = "std"))]
+    use alloc::vec::Vec;
 
     fn assert_writes<T>(data: &[(T, &[u8])])
     where
@@ -263,11 +228,23 @@ mod tests {
             (b"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", b"\x04\x81\x81aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
             (b"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", b"\x04\x82\x01\x02aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
         ]);
+
+        assert_writes::<[u8; 0]>(&[([], b"\x04\x00")]);
+        assert_writes::<[u8; 1]>(&[([1], b"\x04\x01\x01")]);
+        assert_writes::<[u8; 2]>(&[([2, 3], b"\x04\x02\x02\x03")]);
+    }
+
+    #[test]
+    fn test_write_octet_string_encoded() {
+        assert_writes::<OctetStringEncoded<bool>>(&[
+            (OctetStringEncoded::new(true), b"\x04\x03\x01\x01\xff"),
+            (OctetStringEncoded::new(false), b"\x04\x03\x01\x01\x00"),
+        ]);
     }
 
     #[test]
     fn test_write_printable_string() {
-        assert_writes::<PrintableString>(&[
+        assert_writes::<PrintableString<'_>>(&[
             (
                 PrintableString::new("Test User 1").unwrap(),
                 b"\x13\x0bTest User 1",
@@ -281,7 +258,7 @@ mod tests {
 
     #[test]
     fn test_write_ia5string() {
-        assert_writes::<IA5String>(&[
+        assert_writes::<IA5String<'_>>(&[
             (
                 IA5String::new("Test User 1").unwrap(),
                 b"\x16\x0bTest User 1",
@@ -295,7 +272,7 @@ mod tests {
 
     #[test]
     fn test_write_utf8string() {
-        assert_writes::<Utf8String>(&[
+        assert_writes::<Utf8String<'_>>(&[
             (
                 Utf8String::new("Test User 1"),
                 b"\x0c\x0bTest User 1",
@@ -309,7 +286,7 @@ mod tests {
 
     #[test]
     fn test_write_visiblestring() {
-        assert_writes::<VisibleString>(&[
+        assert_writes::<VisibleString<'_>>(&[
             (
                 VisibleString::new("Test User 1").unwrap(),
                 b"\x1a\x0bTest User 1",
@@ -323,7 +300,7 @@ mod tests {
 
     #[test]
     fn test_write_bmpstring() {
-        assert_writes::<BMPString>(&[(
+        assert_writes::<BMPString<'_>>(&[(
             BMPString::new(b"\x00a\x00b\x00c").unwrap(),
             b"\x1e\x06\x00a\x00b\x00c",
         )]);
@@ -331,7 +308,7 @@ mod tests {
 
     #[test]
     fn test_write_universalstring() {
-        assert_writes::<UniversalString>(&[(
+        assert_writes::<UniversalString<'_>>(&[(
             UniversalString::new(b"\x00\x00\x00a\x00\x00\x00b\x00\x00\x00c").unwrap(),
             b"\x1c\x0c\x00\x00\x00a\x00\x00\x00b\x00\x00\x00c",
         )]);
@@ -351,6 +328,14 @@ mod tests {
         ]);
     }
 
+    #[test]
+    fn test_write_u64() {
+        assert_writes::<u64>(&[(
+            12356915591483590945,
+            b"\x02\x09\x00\xab\x7c\x95\x42\xbd\xdd\x89\x21",
+        )]);
+    }
+
     #[test]
     fn test_write_i32() {
         assert_writes::<i32>(&[
@@ -365,6 +350,28 @@ mod tests {
         ]);
     }
 
+    #[test]
+    fn test_write_u16() {
+        assert_writes::<u16>(&[
+            (0, b"\x02\x01\x00"),
+            (1, b"\x02\x01\x01"),
+            (256, b"\x02\x02\x01\x00"),
+            (65535, b"\x02\x03\x00\xff\xff"),
+        ]);
+    }
+
+    #[test]
+    fn test_write_i16() {
+        assert_writes::<i16>(&[
+            (0, b"\x02\x01\x00"),
+            (1, b"\x02\x01\x01"),
+            (-256, b"\x02\x02\xff\x00"),
+            (-1, b"\x02\x01\xff"),
+            (-32768, b"\x02\x02\x80\x00"),
+            (32767, b"\x02\x02\x7f\xff"),
+        ]);
+    }
+
     #[test]
     fn test_write_u8() {
         assert_writes::<u8>(&[
@@ -386,7 +393,7 @@ mod tests {
 
     #[test]
     fn test_write_biguint() {
-        assert_writes::<BigUint>(&[
+        assert_writes::<BigUint<'_>>(&[
             (BigUint::new(b"\x00\xff").unwrap(), b"\x02\x02\x00\xff"),
             (
                 BigUint::new(b"\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff").unwrap(),
@@ -395,9 +402,24 @@ mod tests {
         ]);
     }
 
+    #[test]
+    fn test_write_ownedbiguint() {
+        assert_writes::<OwnedBigUint>(&[
+            (
+                OwnedBigUint::new(b"\x00\xff".to_vec()).unwrap(),
+                b"\x02\x02\x00\xff",
+            ),
+            (
+                OwnedBigUint::new(b"\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff".to_vec())
+                    .unwrap(),
+                b"\x02\x0d\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff",
+            ),
+        ]);
+    }
+
     #[test]
     fn test_write_bigint() {
-        assert_writes::<BigInt>(&[
+        assert_writes::<BigInt<'_>>(&[
             (BigInt::new(b"\xff").unwrap(), b"\x02\x01\xff"),
             (
                 BigInt::new(b"\xff\x7f\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff").unwrap(),
@@ -406,6 +428,18 @@ mod tests {
         ]);
     }
 
+    #[test]
+    fn test_write_ownedbigint() {
+        assert_writes::<OwnedBigInt>(&[
+            (OwnedBigInt::new(b"\xff".to_vec()).unwrap(), b"\x02\x01\xff"),
+            (
+                OwnedBigInt::new(b"\xff\x7f\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff".to_vec())
+                    .unwrap(),
+                b"\x02\x0c\xff\x7f\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff",
+            ),
+        ]);
+    }
+
     #[test]
     fn test_write_object_identifier() {
         assert_writes::<ObjectIdentifier>(&[
@@ -434,7 +468,7 @@ mod tests {
 
     #[test]
     fn test_write_bit_string() {
-        assert_writes::<BitString>(&[
+        assert_writes::<BitString<'_>>(&[
             (BitString::new(b"", 0).unwrap(), b"\x03\x01\x00"),
             (BitString::new(b"\x80", 7).unwrap(), b"\x03\x02\x07\x80"),
             (
@@ -460,34 +494,70 @@ mod tests {
     fn test_write_utctime() {
         assert_writes::<UtcTime>(&[
             (
-                UtcTime::new(Utc.ymd(1991, 5, 6).and_hms(23, 45, 40)).unwrap(),
+                UtcTime::new(DateTime::new(1991, 5, 6, 23, 45, 40).unwrap()).unwrap(),
                 b"\x17\x0d910506234540Z",
             ),
             (
-                UtcTime::new(Utc.timestamp(0, 0)).unwrap(),
+                UtcTime::new(DateTime::new(1970, 1, 1, 0, 0, 0).unwrap()).unwrap(),
                 b"\x17\x0d700101000000Z",
             ),
             (
-                UtcTime::new(Utc.timestamp(1258325776, 0)).unwrap(),
+                UtcTime::new(DateTime::new(2009, 11, 15, 22, 56, 16).unwrap()).unwrap(),
                 b"\x17\x0d091115225616Z",
             ),
         ]);
     }
 
+    #[test]
+    fn test_write_x509_generalizedtime() {
+        assert_writes(&[
+            (
+                X509GeneralizedTime::new(DateTime::new(1991, 5, 6, 23, 45, 40).unwrap()).unwrap(),
+                b"\x18\x0f19910506234540Z",
+            ),
+            (
+                X509GeneralizedTime::new(DateTime::new(1970, 1, 1, 0, 0, 0).unwrap()).unwrap(),
+                b"\x18\x0f19700101000000Z",
+            ),
+            (
+                X509GeneralizedTime::new(DateTime::new(2009, 11, 15, 22, 56, 16).unwrap()).unwrap(),
+                b"\x18\x0f20091115225616Z",
+            ),
+        ]);
+    }
+
     #[test]
     fn test_write_generalizedtime() {
         assert_writes(&[
             (
-                GeneralizedTime::new(Utc.ymd(1991, 5, 6).and_hms(23, 45, 40)).unwrap(),
-                b"\x18\x0f19910506234540Z",
+                GeneralizedTime::new(DateTime::new(1991, 5, 6, 23, 45, 40).unwrap(), Some(1_234))
+                    .unwrap(),
+                b"\x18\x1919910506234540.000001234Z",
             ),
             (
-                GeneralizedTime::new(Utc.timestamp(0, 0)).unwrap(),
+                GeneralizedTime::new(DateTime::new(1991, 5, 6, 23, 45, 40).unwrap(), Some(1))
+                    .unwrap(),
+                b"\x18\x1919910506234540.000000001Z",
+            ),
+            (
+                GeneralizedTime::new(DateTime::new(1970, 1, 1, 0, 0, 0).unwrap(), None).unwrap(),
                 b"\x18\x0f19700101000000Z",
             ),
             (
-                GeneralizedTime::new(Utc.timestamp(1258325776, 0)).unwrap(),
-                b"\x18\x0f20091115225616Z",
+                GeneralizedTime::new(
+                    DateTime::new(2009, 11, 15, 22, 56, 16).unwrap(),
+                    Some(100_000_000),
+                )
+                .unwrap(),
+                b"\x18\x1120091115225616.1Z",
+            ),
+            (
+                GeneralizedTime::new(
+                    DateTime::new(2009, 11, 15, 22, 56, 16).unwrap(),
+                    Some(999_999_999),
+                )
+                .unwrap(),
+                b"\x18\x1920091115225616.999999999Z",
             ),
         ]);
     }
@@ -504,14 +574,16 @@ mod tests {
     fn test_write_sequence() {
         assert_eq!(
             write(|w| {
-                w.write_element(&SequenceWriter::new(&|w: &mut Writer| w.write_element(&())))
+                w.write_element(&SequenceWriter::new(&|w: &mut Writer<'_>| {
+                    w.write_element(&())
+                }))
             })
             .unwrap(),
             b"\x30\x02\x05\x00"
         );
         assert_eq!(
             write(|w| {
-                w.write_element(&SequenceWriter::new(&|w: &mut Writer| {
+                w.write_element(&SequenceWriter::new(&|w: &mut Writer<'_>| {
                     w.write_element(&true)
                 }))
             })
@@ -520,35 +592,35 @@ mod tests {
         );
 
         assert_writes(&[(
-            parse_single::<Sequence>(b"\x30\x06\x01\x01\xff\x02\x01\x06").unwrap(),
+            parse_single::<Sequence<'_>>(b"\x30\x06\x01\x01\xff\x02\x01\x06").unwrap(),
             b"\x30\x06\x01\x01\xff\x02\x01\x06",
         )]);
     }
 
     #[test]
     fn test_write_sequence_of() {
-        assert_writes::<SequenceOfWriter<u8, &[u8]>>(&[
+        assert_writes::<SequenceOfWriter<'_, u8, &[u8]>>(&[
             (SequenceOfWriter::new(&[]), b"\x30\x00"),
             (
                 SequenceOfWriter::new(&[1u8, 2, 3]),
                 b"\x30\x09\x02\x01\x01\x02\x01\x02\x02\x01\x03",
             ),
         ]);
-        assert_writes::<SequenceOfWriter<u8, Vec<u8>>>(&[
+        assert_writes::<SequenceOfWriter<'_, u8, Vec<u8>>>(&[
             (SequenceOfWriter::new(vec![]), b"\x30\x00"),
             (
                 SequenceOfWriter::new(vec![1u8, 2, 3]),
                 b"\x30\x09\x02\x01\x01\x02\x01\x02\x02\x01\x03",
             ),
         ]);
-        assert_writes::<SequenceOfWriter<SequenceWriter, &[SequenceWriter]>>(&[
+        assert_writes::<SequenceOfWriter<'_, SequenceWriter<'_>, &[SequenceWriter<'_>]>>(&[
             (SequenceOfWriter::new(&[]), b"\x30\x00"),
             (
                 SequenceOfWriter::new(&[SequenceWriter::new(&|_w| Ok(()))]),
                 b"\x30\x02\x30\x00",
             ),
             (
-                SequenceOfWriter::new(&[SequenceWriter::new(&|w: &mut Writer| {
+                SequenceOfWriter::new(&[SequenceWriter::new(&|w: &mut Writer<'_>| {
                     w.write_element(&1u64)
                 })]),
                 b"\x30\x05\x30\x03\x02\x01\x01",
@@ -556,14 +628,14 @@ mod tests {
         ]);
 
         assert_writes(&[(
-            parse_single::<SequenceOf<u64>>(b"\x30\x06\x02\x01\x05\x02\x01\x07").unwrap(),
+            parse_single::<SequenceOf<'_, u64>>(b"\x30\x06\x02\x01\x05\x02\x01\x07").unwrap(),
             b"\x30\x06\x02\x01\x05\x02\x01\x07",
         )]);
     }
 
     #[test]
     fn test_write_set_of() {
-        assert_writes::<SetOfWriter<u8, &[u8]>>(&[
+        assert_writes::<SetOfWriter<'_, u8, &[u8]>>(&[
             (SetOfWriter::new(&[]), b"\x31\x00"),
             (SetOfWriter::new(&[1u8]), b"\x31\x03\x02\x01\x01"),
             (
@@ -589,73 +661,25 @@ mod tests {
         ]);
 
         assert_writes(&[(
-            parse_single::<SetOf<u64>>(b"\x31\x06\x02\x01\x05\x02\x01\x07").unwrap(),
+            parse_single::<SetOf<'_, u64>>(b"\x31\x06\x02\x01\x05\x02\x01\x07").unwrap(),
             b"\x31\x06\x02\x01\x05\x02\x01\x07",
         )]);
     }
 
     #[test]
     fn test_write_implicit() {
-        #[cfg(feature = "const-generics")]
         assert_writes::<Implicit<bool, 2>>(&[
             (Implicit::new(true), b"\x82\x01\xff"),
             (Implicit::new(false), b"\x82\x01\x00"),
         ]);
-
-        assert_eq!(
-            write(|w| { w.write_optional_implicit_element(&Some(true), 2) }).unwrap(),
-            b"\x82\x01\xff"
-        );
-        assert_eq!(
-            write(|w| { w.write_optional_explicit_element::<u8>(&None, 2) }).unwrap(),
-            b""
-        );
-
-        assert_eq!(
-            write(|w| {
-                w.write_optional_implicit_element(&Some(SequenceWriter::new(&|_w| Ok(()))), 2)
-            })
-            .unwrap(),
-            b"\xa2\x00"
-        );
-        assert_eq!(
-            write(|w| { w.write_optional_explicit_element::<SequenceWriter>(&None, 2) }).unwrap(),
-            b""
-        );
-
-        assert_eq!(
-            write(|w| { w.write_implicit_element(&true, 2) }).unwrap(),
-            b"\x82\x01\xff"
-        );
-
-        assert_eq!(
-            write(|w| { w.write_implicit_element(&SequenceWriter::new(&|_w| { Ok(()) }), 2) })
-                .unwrap(),
-            b"\xa2\x00"
-        );
     }
 
     #[test]
     fn test_write_explicit() {
-        #[cfg(feature = "const-generics")]
         assert_writes::<Explicit<bool, 2>>(&[
             (Explicit::new(true), b"\xa2\x03\x01\x01\xff"),
             (Explicit::new(false), b"\xa2\x03\x01\x01\x00"),
         ]);
-
-        assert_eq!(
-            write(|w| { w.write_optional_explicit_element(&Some(true), 2) }).unwrap(),
-            b"\xa2\x03\x01\x01\xff"
-        );
-        assert_eq!(
-            write(|w| { w.write_optional_explicit_element::<u8>(&None, 2) }).unwrap(),
-            b""
-        );
-
-        assert_eq!(
-            write(|w| { w.write_explicit_element(&true, 2) }).unwrap(),
-            b"\xa2\x03\x01\x01\xff"
-        );
     }
 
     #[test]
@@ -687,17 +711,31 @@ mod tests {
     fn test_write_tlv() {
         assert_writes(&[
             (
-                parse_single::<Tlv>(b"\x01\x01\x00").unwrap(),
+                parse_single::<Tlv<'_>>(b"\x01\x01\x00").unwrap(),
                 b"\x01\x01\x00",
             ),
             (
-                parse_single::<Tlv>(b"\x1f\x81\x80\x01\x00").unwrap(),
+                parse_single::<Tlv<'_>>(b"\x1f\x81\x80\x01\x00").unwrap(),
                 b"\x1f\x81\x80\x01\x00",
             ),
             (
-                parse_single::<Tlv>(b"\x1f\x1f\x00").unwrap(),
+                parse_single::<Tlv<'_>>(b"\x1f\x1f\x00").unwrap(),
                 b"\x1f\x1f\x00",
             ),
         ]);
     }
+
+    #[test]
+    fn test_write_box() {
+        assert_writes(&[
+            (Box::new(12u8), b"\x02\x01\x0c"),
+            (Box::new(0), b"\x02\x01\x00"),
+        ]);
+    }
+
+    #[test]
+    fn test_write_error_display() {
+        use alloc::string::ToString;
+        assert_eq!(&WriteError::AllocationError.to_string(), "allocation error");
+    }
 }
